//
// "$Id: main.cxx,v 1.11 2002/11/11 04:32:32 easysw Exp $"
//
// FLTK photo program main entry.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// Contents:
//
//   main() - Main entry for flphoto.
//

#include "flphoto.h"
#include "Fl_CRW_Image.H"
#include "Fl_PCD_Image.H"
#include <FL/filename.H>
#include "config.h"


//
// 'main()' - Main entry for flphoto.
//

int					// O - Exit status
main(int  argc,				// I - Number of command-line arguments
     char *argv[])			// I - Command-line arguments
{
  int		i;			// Looping var
  flphoto	*app = 0,		// Current application window
		*album = 0;		// Current album window


  // Register all image formats...
  fl_register_images();
  Fl_Shared_Image::add_handler(Fl_PCD_Image::check);
  Fl_Shared_Image::add_handler(Fl_CRW_Image::check);

  // Set the UI scheme, defaulting to "plastic" because it looks better.
  Fl::get_system_colors();

  if (getenv("FLTK_SCHEME"))
    Fl::scheme(NULL);
  else
    Fl::scheme("plastic");

  // Loop through the command-line looking for albums, files, and
  // directories.
  for (i = 1; i < argc; i ++)
  {
    if (fl_filename_match(argv[i], "*.album"))
    {
      album = new flphoto(argv[i]);
      album->show();
    }
    else if (!app)
    {
      app = new flphoto(argv[i]);
      app->show();
    }
    else if (fl_filename_isdir(argv[i]))
      app->open_dir(argv[i]);
    else
      app->open_file(argv[i]);
  }

  // Create an empty album if no files are provided on the command-line.
  if (!app && !album)
  {
    app = new flphoto();
    app->show();
  }

  // Optionally show the license agreement...
  flphoto::prefs.get("license_version", i, 0);

  if (i < FLPHOTO_VERNUMBER)
  {
    flphoto::prefs.set("license_version", FLPHOTO_VERNUMBER);
    flphoto::help_cb("license.html");
  }

  // Run the app until the user quits...
  return (Fl::run());
}


//
// End of "$Id: main.cxx,v 1.11 2002/11/11 04:32:32 easysw Exp $".
//
