//
// "$Id: slideshow.cxx,v 1.10 2002/12/02 03:14:08 easysw Exp $"
//
// Slideshow methods.
//
// Copyright 2002 by Michael Sweet.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// Contents:
//
//   flphoto::slideshow_cb() - Show a slideshow.
//   flphoto::slidenext_cb() - Show the next image...
//

#include "flphoto.h"
#include <FL/Fl_Button.H>


//
// Local globals...
//

static flphoto		*current_album;
static int		current_image;
static Fl_Double_Window	*window = 0;
static Fl_Image_Display	*display = 0;


//
// 'flphoto::slideshow_cb()' - Show a slideshow.
//

void
flphoto::slideshow_cb()
{
  int	val;					// Preference value


  // See if we have any images in this album...
  if (!browser_->count())
  {
    fl_alert("No images to show in a slideshow!");
    return;
  }

  // Save the current album...
  current_album = this;
  current_image = -1;

  // Get the saved slideshow preferences...
  prefs.get("slideshow_delay", val, 5);
  ssoptions_delay_value_->value(val);

  prefs.get("slideshow_repeat", val, 1);
  ssoptions_repeat_button_->value(val);

  // Show the preferences dialog and wait for the user to click "Continue"...
  ssoptions_window_->hotspot(ssoptions_window_);
  ssoptions_window_->show();

  while (ssoptions_window_->shown())
    Fl::wait();

  // Save the slideshow settings...
  prefs.set("slideshow_delay", (int)ssoptions_delay_value_->value());
  prefs.set("slideshow_repeat", ssoptions_repeat_button_->value());

  // Create the slideshow window as needed...
  if (!window)
  {
    window = new Fl_Double_Window(0, 0, Fl::w(), Fl::h());
    window->modal();

    display = new Fl_Image_Display(0, 0, Fl::w() + 17, Fl::h() + 17);
    display->box(FL_FLAT_BOX);
    display->color(FL_BLACK);
    display->callback((Fl_Callback *)slidenext_cb);
  }

  // Clear the slideshow window and then show the next image...
  display->image(0);
  window->show();

  slidenext_cb();

  // Wait until the user closes the slideshow...
  while (window->shown())
    Fl::wait();

  // Remove any pending timeouts...
  Fl::remove_timeout((Fl_Timeout_Handler)slidenext_cb);

  // Show the current image in the window...
  open_image_cb(current_image);
}


//
// 'flphoto::slidenext_cb()' - Show the next image...
//

void
flphoto::slidenext_cb()
{
  Fl_Image_Browser::ITEM *item;			// Item in image browser...


  // Remove any prior slideshow timeouts...
  Fl::remove_timeout((Fl_Timeout_Handler)slidenext_cb);

  // If the window is no longer visible, then we're done!
  if (!window->shown())
    return;

  // Loop until we find the next image...
  for (;;)
  {
    // Unload the current image, if any...
    if (current_image >= 0)
    {
      item = current_album->browser_->value(current_image);

      if (item->image && !item->changed && item != current_album->image_item_)
      {
        item->image->release();
	item->image = 0;
      }
    }

    // Advance to the next or previous image...
    if (Fl::event_button() == FL_RIGHT_MOUSE ||
        Fl::event_key() == FL_BackSpace)
      current_image --;
    else
      current_image ++;

    // Wrap to the beginning or end, as needed...
    if (current_image >= current_album->browser_->count())
    {
      current_image = 0;

      if (!current_album->ssoptions_repeat_button_->value())
      {
        // Don't go back to the first image...
        window->hide();
        return;
      }
    }
    else if (current_image < 0)
      current_image = current_album->browser_->count() - 1;

    // Load the current image...
    window->cursor(FL_CURSOR_WAIT);
    Fl::flush();

    display->value(current_album->browser_->load_item(current_image));
    Fl::flush();

    // Pre-load next image...
    if ((current_image + 1) < current_album->browser_->count())
      current_album->browser_->load_item(current_image + 1);
    else
      current_album->browser_->load_item(0);

    window->cursor(FL_CURSOR_DEFAULT);

    break;
  }

  // Set a timer for the next image...
  if (current_album->ssoptions_delay_value_->value() > 0.0)
    Fl::add_timeout(current_album->ssoptions_delay_value_->value(),
                    (Fl_Timeout_Handler)slidenext_cb);
}


//
// End of "$Id: slideshow.cxx,v 1.10 2002/12/02 03:14:08 easysw Exp $".
//
