/*****************************************************************************\
  DJ3320.h : Interface for the DJ3320 class

  Copyright (c) 2001 - 2002, Hewlett-Packard Co.
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  3. Neither the name of Hewlett-Packard nor the names of its
     contributors may be used to endorse or promote products derived
     from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
  NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
  TO, PATENT INFRINGEMENT; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  \***************************************************************************/


#ifndef APDK_DJ3320_H
#define APDK_DJ3320_H

#include "config.h"

APDK_BEGIN_NAMESPACE

#include "ldlencap.h"

class   LDLEncap;

class DJ3320 : public Printer
{
public:
    DJ3320 (SystemServices* pSS, BOOL proto = FALSE);

    virtual ~DJ3320 ();

    Header* SelectHeader(PrintContext* pc);
//    Compressor* CreateCompressor(unsigned int RasterSize);
    virtual DRIVER_ERROR Send (const BYTE *pWriteBuff, DWORD dwWriteLen);
    virtual DRIVER_ERROR Encapsulate (const BYTE *pRasterData, DWORD size, BOOL bLastPlane);
    DRIVER_ERROR VerifyPenInfo ();
    virtual DRIVER_ERROR ParsePenInfo (PEN_TYPE& ePen, BOOL QueryPrinter = TRUE);
    virtual DRIVER_ERROR SkipRasters (int nBlankRasters);
    virtual DRIVER_ERROR CleanPen ();
    virtual DRIVER_ERROR Flush (int FlushSize);
    DRIVER_ERROR SetPens (PEN_TYPE eNewPen);

    DISPLAY_STATUS ParseError (BYTE byStatusReg);

    LDLEncap    *pLDLEncap;

protected:
    DISPLAY_STATUS m_dsCurrentStatus;
    void    InitPrintModes ();
};

class CrossBowGrayMode : public PrintMode
{
public:
    CrossBowGrayMode (PEN_TYPE ePen);
};

class CrossBowKDraftMode : public GrayMode
{
public:
    CrossBowKDraftMode ();
};

class CrossBowDraftMode : public PrintMode
{
public:
    CrossBowDraftMode (PEN_TYPE ePen);

};

class CrossBowNormalMode : public PrintMode
{
public:
    CrossBowNormalMode (PEN_TYPE ePen);

};

class CrossBowPhotoMode : public PrintMode
{
public:
    CrossBowPhotoMode ();
};

#define SWATH_HEIGHT    100     // Number of color nozzles to be used

class LDLEncap //: public Compressor
{
public:
    LDLEncap (DJ3320 *pPrinter, SystemServices *pSys, PrintContext *pc);
    DRIVER_ERROR Encapsulate (const BYTE* input, DWORD size, BOOL bLastPlane);
    ~LDLEncap ();

    DRIVER_ERROR StartJob ();
    DRIVER_ERROR EndPage ();
    DRIVER_ERROR EndJob ();
    DRIVER_ERROR Continue ();
    DRIVER_ERROR CleanPen ();
    void    Cancel ();
    void    Flush ();
    DRIVER_ERROR    SetVerticalSkip (int nBlankRasters);
    void AllocateSwathBuffer (unsigned int RasterSize);

    BOOL UpdateState (BOOL bInitialize);

    DJ3320  *pPrinterXBow;
    DRIVER_ERROR    constructor_error;

    short int   *piCreditCount;
    BYTE        byNumberOfCommands;
    BYTE        byStatusBuff[DevIDBuffSize];
    BOOL        bNewStatus;
//  BYTE        byQuery[QUERYSIZE];
    BYTE        pbyCancel[CANCELSIZE];
    BYTE        *pbySync;

private:
    DRIVER_ERROR    StartPage ();

    SystemServices  *m_pSys;
    int     m_iRasterCount;
    int     m_iImageWidth;
    int     m_iXResolution;
    int     m_iYResolution;
    UInt16  m_sRefCount;
    int     m_iNumColors;
    char    m_cPrintDirection;
    BYTE    *m_szCmdBuf;
    BOOL    m_bLittleEndian;
    PrintContext    *m_pthisPC;
    BYTE    ***m_SwathData;
    short   m_sSwathHeight;
    int     m_iBlankRasters;
    int     m_iVertPosn;
    int     m_iLeftMargin;
    BOOL    m_bBidirectionalPrintingOn;
    Int16   m_iBitDepth;
    BYTE    m_cPassNumber;
    BYTE    m_cPlaneNumber;
    BYTE    m_cPrintQuality;
    BYTE    m_cMediaType;
    BYTE    *m_szCompressBuf;
    int     m_iNextRaster;
    int     m_iNextColor;

	BOOL	m_bStartPageNotSent;

    UInt16  m_uVal16;
    UInt32  m_uVal32;
    BYTE    *m_cByte;
    BYTE    *m_pbyPacketBuff;
	DWORD	m_dwPacketBuffSize;

    DRIVER_ERROR    PrintSweep (UInt32 SweepSize,
                        BOOL ColorPresent,
                        BOOL BlackPresent,
                        Int32 VerticalPosition,
                        Int32 LeftEdge,
                        Int32 RightEdge,
                        char PrintDirection,
                        Int16 sFirstNozzle,
                        Int16 sLastNozzle = 0);
    DRIVER_ERROR    LoadSweepData (BYTE *dataPtr, int imagesize);
    void            FillLidilHeader (void *pLidilHdr, int Command,
                             UInt16 CmdLen, UInt16 DataLen);
    unsigned int    GetSwathWidth (int iFirst, int iLast, int iWidth);
    BOOL            IsBlankRaster (BYTE *input, int iWidth);
    DRIVER_ERROR    ProcessSwath (int iCurRasterWidth);

    BOOL            GetPackets (DWORD &dwBytesRead);
};

#define WRITE16(value)  \
        m_uVal16 = value; \
        m_cByte = (BYTE *) &m_uVal16; \
        if (m_bLittleEndian)       \
        { \
            m_szCmdBuf[index++] = m_cByte[1];  \
            m_szCmdBuf[index++] = m_cByte[0]; \
        } \
        else \
        { \
            memcpy (m_szCmdBuf+index, m_cByte, 2); \
            index += 2; \
        }

#define WRITE32(value) \
        m_uVal32 = value; \
        m_cByte = (BYTE *) &m_uVal32; \
        if (m_bLittleEndian)       \
        { \
            m_szCmdBuf[index++] = m_cByte[3];  \
            m_szCmdBuf[index++] = m_cByte[2]; \
            m_szCmdBuf[index++] = m_cByte[1]; \
            m_szCmdBuf[index++] = m_cByte[0]; \
        } \
        else \
        { \
            memcpy (m_szCmdBuf+index, m_cByte, 4); \
            index += 4; \
        }

#ifdef APDK_PROTO
extern PEN_TYPE ProtoPenType;
class ProtoDJ3320 : public DJ3320
{
public:
    ProtoDJ3320 (ProtoServices* pSS);

    DRIVER_ERROR ParsePenInfo(PEN_TYPE& ePen){ ePen=ProtoPenType; return NO_ERROR; }
    BOOL PhotoTrayInstalled();
};


#endif

APDK_END_NAMESPACE

#endif //APDK_DJ3320_H
